--- old/usr/src/uts/common/fs/zfs/spa.c	2013-04-24 17:38:58.253454815 -0600
+++ new/usr/src/uts/common/fs/zfs/spa.c	2013-04-24 17:38:58.263722765 -0600
@@ -77,6 +77,12 @@
 #include "zfs_prop.h"
 #include "zfs_comutil.h"
 
+/*
+ * The interval at which failed configuration cache file writes
+ * should be retried.
+ */
+static int zfs_ccw_retry_interval = 300;
+
 typedef enum zti_modes {
 	ZTI_MODE_FIXED,			/* value is # of threads (min 1) */
 	ZTI_MODE_ONLINE_PERCENT,	/* value is % of online CPUs */
@@ -5661,13 +5667,33 @@
 	mutex_exit(&spa->spa_async_lock);
 }
 
+static int
+spa_async_tasks_pending(spa_t *spa)
+{
+	u_int non_config_tasks;
+	u_int config_task;
+	boolean_t config_task_suspended;
+
+	non_config_tasks = spa->spa_async_tasks & ~SPA_ASYNC_CONFIG_UPDATE;
+	config_task = spa->spa_async_tasks & SPA_ASYNC_CONFIG_UPDATE;
+	if (spa->spa_ccw_fail_time == 0)
+		config_task_suspended = B_FALSE;
+	else
+		config_task_suspended =
+		    (ddi_get_lbolt64() - spa->spa_ccw_fail_time)
+		  < (zfs_ccw_retry_interval * hz);
+
+	return (non_config_tasks || (config_task && !config_task_suspended));
+}
+
 static void
 spa_async_dispatch(spa_t *spa)
 {
 	mutex_enter(&spa->spa_async_lock);
-	if (spa->spa_async_tasks && !spa->spa_async_suspended &&
+	if (spa_async_tasks_pending(spa) &&
+	    !spa->spa_async_suspended &&
 	    spa->spa_async_thread == NULL &&
-	    rootdir != NULL && !vn_is_readonly(rootdir))
+	    rootdir != NULL)
 		spa->spa_async_thread = thread_create(NULL, 0,
 		    spa_async_thread, spa, 0, &p0, TS_RUN, maxclsyspri);
 	mutex_exit(&spa->spa_async_lock);
